#include "Gardener.h"

//Descr: create a gardener 
//In: -
//Out: an empty instance of Gardener
Gardener::Gardener(){
	name = NULL;
	noFlowers = 0;
	flowers = new Flower[MAX_FLOWERS];
}

//Descr: create a gardener 
//In: name, no of flowers, array of flowers
//Out: an instance of Gardener
Gardener::Gardener(char* n, int nF, Flower* f){
	name = new char[strlen(n) + 1];
	strcpy_s(name, strlen(n) + 1, n);
	noFlowers = nF;
	flowers = new Flower[nF];
	for(int i = 0; i <  noFlowers; i++)
		flowers[i] = f[i];
}

//Descr: create a gardener 
//In: a string that must be parsed
//Out: an instance of Gardener
Gardener::Gardener(char* s){
	//to do
}

//Descr: distroy a gardener 
//In: a gardener 
//Out: - (destructor)
Gardener::~Gardener(){
	if (name){
		delete[] name;
		name = NULL;
	}
	if (flowers){
		delete[] flowers;
		flowers = NULL;
	}
}

//Descr: create a gardener using info from another gardener
//In: a gardener g
//Out: an instance of Gardener with info from g
Gardener::Gardener(const Gardener &g){
	name = new char[strlen(g.name) + 1];
	strcpy_s(name, strlen(g.name) + 1, g.name);
	noFlowers = g.noFlowers;
	flowers = new Flower[noFlowers];
	for(int i = 0; i < noFlowers; i++)
			flowers[i] = g.flowers[i];
}

//Descr: create a new Gardener (equal to a given gardener g)
//In: a gardener g
//Out: a new gardener (equal to g)
Gardener& Gardener::operator=(const Gardener &g){
	if (this != &g){
		if (name)
			delete[] name;
		name = new char[strlen(g.name) + 1];
		strcpy_s(name, strlen(g.name) + 1,g.name);
		noFlowers = g.noFlowers;
		if (flowers)
			delete[] flowers;
		flowers = new Flower[noFlowers];
		for(int i = 0; i < noFlowers; i++)
			flowers[i] = g.flowers[i];
	}
	return *this;
}

//Descr: change the name of a gardener 
//In: a gardener and a name
//Out: the gardener with changed name
void Gardener::setName(char* n){
	if (name)
		delete[] name;
	name = new char[strlen(n) + 1];
	strcpy_s(name, strlen(n) + 1, n);
}

//Descr: change the no of flowers of a gardener 
//In: a gardener and a no
//Out: the gardener with changed no of flowers
void Gardener::setNoFlowers(int n){
	noFlowers = n;
}

//Descr:add a flower for a gardener 
//In: a gardener and a flower
//Out: the gardener with updated array of flowers
void Gardener::addFlower(Flower &f){
	if (noFlowers == MAX_FLOWERS){
		Flower* flowersAux = new Flower[2 * MAX_FLOWERS];
		for(int i = 0; i < noFlowers; i++)
			flowersAux[i] = flowers[i];
		delete[] flowers;
		flowers = flowersAux;
	}
	flowers[noFlowers++] = f;
}

//Descr: access the name of a gardener 
//In: a gardener
//Out: the name
char* Gardener::getName(){
	return name;
}

//Descr: access the no of flowers of a gardener 
//In: a gardener
//Out: the no of flowers
int Gardener::getNoFlowers(){
	return noFlowers;
}

//Descr: access a flower (from a position) of a gardener 
//In: a gardener and a position
//Out: the flower from the given position
Flower& Gardener::getFlower(int pos){
	if ((pos >= 0) && (pos < noFlowers))
		return flowers[pos];
}

//Descr: convert a gardener into a string
//In: a gardener
//Out: a string with info about the gardener
char* Gardener::toString(){
	int l = strlen(name) + noFlowers * (MAX_NAME + 1) + 6;
	char* s = new char[l];
	strcpy_s(s, l, name);
	strcat_s(s, l, ",");
	char* aux = new char[5];

	_itoa_s(this->noFlowers, aux, 5, 10);
	strcat_s(s, l, aux);
	strcat_s(s, l, ":");
	//strcpy(s, "");
	for(int i = 0; i < noFlowers; i++){
		strcat_s(s, l, flowers[i].toString());
	}
	return s;
}

//Descr: compare 2 gardeners (the current one and a new one)
//In: two gardeners
//Out: true/false
bool Gardener::compare(Gardener &g){
	return (strcmp(name, g.name) == 0);
}